import("plate.lua")

Cart = {}

local sizeX, sizeY = CartSprite:getSize()
local size = { x = sizeX, y = sizeY * 0.75 }

function Cart.reset(o)
    o.position = {
        x = 20,
        y = 50,
    }
    o.velocity = {
        x = 50 + (150 * math.random()),
        y = 150 * (math.random() - 1),
    }
    o.size = size
    o.canBeBounced = {
        flat = { x = 0, y = 0 },
        mult = { x = 1, y = 1 },
    }
    o.canCollideWith = 1
    o.mass = 1
    o.drawAsSprite = CartSprite
    o.focusPriority = 0

    ---@type pd_image[]
    o.collectables = {}

    ---@param world World
    function o:spawnEntitiesWhenStopped(world)
        local y = self.position.y - 240
        local rectWidth = 150
        local plateSize = { x = rectWidth, y = 10 }
        self.velocity = { x = 300, y = 0 }
        world:addEntity(self)
        world:addEntity({
            position = { x = self.position.x, y = self.position.y },
            focusPriority = 1,
        })
        world:addEntity({
            drawAsRectangle = { size = plateSize },
            size = plateSize,
            mass = 0.5,
            velocity = { x = 0, y = 0 },
            position = { x = self.position.x - (rectWidth / 2), y = y },
            canCollideWith = 2,
            canBeCollidedBy = 2,
            isSolid = true,
            stopMovingOnCollision = true,
        })

        for _, collectable in ipairs(self.collectables) do
            local collX, collY = collectable:getSize()
            y = y - collY - 5
            world:addEntity({
                drawAsSprite = collectable,
                size = { x = collX, y = collY / 2 },
                mass = 0.5,
                velocity = { x = 0, y = 0 },
                position = { x = self.position.x - (collX / 2), y = y },
                canCollideWith = 2,
                canBeCollidedBy = 2,
                isSolid = true,
                stopMovingOnCollision = true,
            })
        end
    end

    return o
end

function Cart.new()
    return setmetatable(Cart.reset({}), { __index = Cart })
end
